-- -----------------------------------------------------------------------------

local ADDON_NAME, ns = ...
local WFEZ_D,WFEZ_O,WFEZ_F,L = ns.datas,ns.functions,ns.frames,ns.locales

-- -----------------------------------------------------------------------------
-- Debug Functions
-- -----------------------------------------------------------------------------

function WFEZ_O:DEBUG(what,more,bug)
    if not WFEZ_D.DEBUG then return end
    local colors = {['VAR']      = WFEZ_D.colors.yellow,
                    ['CALL']     = WFEZ_D.colors.magenta,
                    ['REGISTER'] = WFEZ_D.colors.red,
                    ['FIRE']     = WFEZ_D.colors.green,
                    ['OPTIONS']  = WFEZ_D.colors.blue,
                    ['TABLE']    = WFEZ_D.colors.pink }
    local color = (colors[what] or WFEZ_D.colors.yellow)

    local res = (WFEZ_D.PRINT.."|cff"..color..(more and tostring(more) or '> ').."|r"..(bug and ' '..tostring(bug) or ''))
    if DLAPI then DLAPI.DebugLog('_DebugLog', res) end
end
function WFEZ_O:DEBUG_T(name,tabl)
    if not WFEZ_D.DEBUG then return end
    local res = " "
    for k,v in pairs(tabl) do
        res = res..k.." |cff"..WFEZ_D.colors.pink.."*|r "
    end
    WFEZ_O:DEBUG('TABLE',name,res)
end
function WFEZ_O:DEBUG_TV(name,tabl)
    if not WFEZ_D.DEBUG then return end
    local res = ""
    for k,v in pairs(tabl) do
        res = res..k.." > "..tostring(v).." |cff"..WFEZ_D.colors.pink.."***|r "
    end
    WFEZ_O:DEBUG('TABLE',name,res)
end

function WFEZ_O:HEX(c)
	c = math.floor(c * 255)
	local hex = string.format("%x", c)
	if (hex:len() == 1) then
		return "0"..hex;
	end
	return hex;
end

function WFEZ_O:TRIM(s)
	if not s then return end
	return s:match'^()%s*$' and '' or s:match'^%s*(.*%S)'
end

-- -----------------------------------------------------------------------------
-- Create Frame
-- -----------------------------------------------------------------------------

function WFEZ_O:FRAM(name,type,where,opt, 	z,x,y,yy,zz, 	w,h)
	if w == nil then w = WFEZ_D.fonts.TXT_14 end
	if h == nil then h = WFEZ_D.fonts.TXT_14 end
    local obj = name
    local stuff = _G[obj] or CreateFrame(type,obj,where,opt)
    if z then stuff:SetPoint(z,x,y,yy,zz) end
    if w and h then stuff:SetSize(w,h) end
    return stuff
end
function WFEZ_O:BTTN(name,where, 			z,x,y,yy,zz, 	w,h)
	local btn = WFEZ_O:FRAM(name,"Button",where,"UIPanelButtonTemplate", z,x,y,yy,zz,w,h)
		  btn:DisableDrawLayer("BACKGROUND")
		  btn:SetNormalTexture('') btn:SetPushedTexture('') btn:SetHighlightTexture('')

		  local font = btn:GetNormalFontObject();
				font:SetTextColor(1,1,1,.6); btn:SetNormalFontObject(font);
		  local font = btn:GetHighlightFontObject();
				font:SetTextColor(1,1,1,1); btn:SetHighlightFontObject(font);

	return btn
end
function WFEZ_O:FADE(name,where,allpoint,f)
    local obj = name
    local stuff = _G[obj] or where:CreateTexture(obj, "BACKGROUND")
    if allpoint then stuff:SetAllPoints(true) end
          stuff:SetColorTexture(0,0,0,f)
    return stuff
end
function WFEZ_O:ICON(name,where,ico,		z,x,y,		w,h)
	if w == nil then w = WFEZ_D.fonts.TXT_14 end
	if h == nil then h = WFEZ_D.fonts.TXT_14 end
    local obj = name
    local stuff = _G[obj] or where:CreateTexture(obj,"HIGH")
    if z then stuff:SetPoint(z,x,y) end
          stuff:SetSize(w,h)
          stuff:SetTexture(ico)
    return stuff
end
function WFEZ_O:FONT(name,where,	z,x,y,yy,zz,	w,h,	font,justifyH)
	if font == nil then font = WFEZ_D.fonts.TXT_F end
	if justifyH == nil then justifyH = "LEFT" end
    local obj = name
    local stuff = _G[obj] or where:CreateFontString(obj,nil,font)
    if z then stuff:SetPoint(z,x,y,yy,zz) end
    if w and h then stuff:SetSize(w,h) end
          stuff:SetJustifyH(justifyH)
          stuff:SetJustifyV("CENTER")
          stuff:SetWordWrap(false)
    return stuff
end

-- -----------------------------------------------------------------------------
-- GET STATES
-- -----------------------------------------------------------------------------

function WFEZ_O:SHOW_RIO(tooltip,fullname,realmName,playerFaction)
    if not RaiderIO then return '' end;

    local FACTIONS = { Alliance = 1, Horde = 2, Neutral = 3 }
    local playerFactionID = FACTIONS[playerFaction or UnitFactionGroup("player")]
    RaiderIO.ShowProfile(tooltip,fullname, realmName, playerFactionID);
end
function WFEZ_O:GET_RIO(fullname,realmName,playerFaction)
    if not RaiderIO then return '' end;

    local FACTIONS = { Alliance = 1, Horde = 2, Neutral = 3 }
    local playerFactionID = FACTIONS[playerFaction or UnitFactionGroup("player")]
    local playerProfile = RaiderIO.GetProfile(fullname, realmName, playerFactionID);
    local currentScore = '';
    local previousScore = '';

    if (playerProfile ~= nil) then
        if playerProfile.mythicKeystoneProfile ~= nil then
            currentScore = playerProfile.mythicKeystoneProfile.currentScore or '';
            -- previousScore = playerProfile.mythicKeystoneProfile.previousScore or 0;
        end
    end
    -- score = currentScore
	if currentScore == '' then return '' end
	local r, g, b = RaiderIO.GetScoreColor(currentScore);
    local hex = WFEZ_O:HEX(r)..WFEZ_O:HEX(g)..WFEZ_O:HEX(b);
	return "|cff"..hex..currentScore.." |r ";

end

function WFEZ_O:GET_COLOR(className)
	local clname = (className and className ~= '') and WF_ez_DBC.UnlocalizedClasses[className]
	return RAID_CLASS_COLORS[clname]
end

-- -----------------------------------------------------------------------------
-- LDB LIBSTUB
-- -----------------------------------------------------------------------------

function WFEZ_O:UPDATE_LDB()
	if WFEZ_F.ldb or WFEZ_D.isOpen then
    	WFEZ_O:DEBUG('RegisterEvent','UPDATE_LDB')
		local guildies = ""
		local ON_Friends = C_FriendList.GetNumOnlineFriends()
		local NB_Bnfrnds, ON_Bnfrnds = BNGetNumFriends()
		local NB_Friends = C_FriendList.GetNumFriends()

		local datatext = ""
		if IsInGuild() then
			local numTotal, numOnline, numOnlineAndMobile = GetNumGuildMembers();
			for i = 1, numTotal do
				local name, _, _, _, _, _, _, _, isOnline, _, className, _, _, isMobile, _, _, _ = GetGuildRosterInfo(i)
				if not name then return end
				if isMobile or isOnline then
					local playername,playerrealm =  strsplit( "-",name, 2 ) or ""
					local c = RAID_CLASS_COLORS[className] or 'FFF4F4F4' -- WFEZ_O:GET_COLOR(className)
					guildies =guildies.." |c"..c.colorStr..playername.."|r ";
				end
			end
			if WFEZ_D.playernme == "Woøtzee" then
				datatext = 'F '..(ON_Friends+ON_Bnfrnds)..' # |cFFffc600G '..numOnline..'|r [ '..guildies..' ]'
			else
				datatext = 'FRIENDS:|cFFffc600'..(ON_Friends+ON_Bnfrnds)..'|r   GUILD:|cFFffc600'..numOnline..'|r'..(guildies~="" and '  '..guildies or '')
			end
			if WF_ez_DBC.showguildie then WFEZ_D.TXTUP = 'GUILD: |cFFB4B4B4'..numOnline..'|r/'..numTotal..'   FRIENDS: |cFFB4B4B4'..(ON_Friends+ON_Bnfrnds)..'|r/'..(NB_Friends+NB_Bnfrnds)
									 else WFEZ_D.TXTUP = 'FRIENDS: |cFFB4B4B4'..(ON_Friends+ON_Bnfrnds)..'|r/'..(NB_Friends+NB_Bnfrnds) end
		else
			    datatext = 'FRIENDS: |cFFffc600'..(ON_Friends+ON_Bnfrnds)..'|r/'..(NB_Friends+NB_Bnfrnds)
			WFEZ_D.TXTUP = 'FRIENDS: |cFFB4B4B4'..(ON_Friends+ON_Bnfrnds)..'|r/'..(NB_Friends+NB_Bnfrnds)
		end
		if WFEZ_D.FAKE then
			guildies = WFEZ_O:FAKE_GUILDIES()
			datatext = 'FRIENDS:|cFFffc600'..(38)..'|r   GUILD:|cFFffc600'..(9)..'|r  '..guildies
			WFEZ_D.TXTUP = 'GUILD: |cFFB4B4B4'..(9)..'|r/'..(156)..'   FRIENDS: |cFFB4B4B4'..(18)..'|r/'..(132)
		end
		if WFEZ_F.ldb then WFEZ_F.dataobj.text = datatext end
		if _G['WFEZ_TXT'] and WFEZ_D.isOpen then _G['WFEZ_TXT']:SetText(WFEZ_D.TXTUP) end
	end
end

function WFEZ_O:UPDATE_MAP()
	if WFEZ_F.ldb then
    	WFEZ_O:DEBUG('RegisterEvent','UPDATE_MAP')
		local icon = LibStub("LibDBIcon-1.0", true)
		if not icon then return end
		local WFEZ_dataobj = WFEZ_F.ldb:NewDataObject("FRIENDS",{
			type = "launcher",
			icon = "Interface\\ChatFrame\\UI-ChatIcon-Battlenet",
			OnClick = function() WFEZ_O:CHECK_OPEN() end,
		})
		WF_ez_DBC.dataobjpos = WF_ez_DBC.dataobjpos or { 90 }
		icon:Register("WOEZ FRIENDS", WFEZ_dataobj, WF_ez_DBC.dataobjpos)
	end
end

-- -----------------------------------------------------------------------------
-- BUILD FRIENDS
-- -----------------------------------------------------------------------------

function WFEZ_O:GET_FRIENDS()
    if WFEZ_D.isOpen then
    	WFEZ_O:DEBUG('VAR','GET_FRIENDS')
		wipe(WFEZ_D.friendsAl)
		-- local ON_Friends = C_FriendList.GetNumOnlineFriends()
		if WF_ez_DBC.showguildie then WFEZ_O:GET_GUILD_TABLE() end
		local NB_Bnfrnds, ON_Bnfrnds = BNGetNumFriends()
		local NB_Friends = C_FriendList.GetNumFriends()
		if NB_Bnfrnds > 0 then WFEZ_O:GET_FRIENDS_TABLE(NB_Bnfrnds,true) end
		if NB_Friends > 0 then WFEZ_O:GET_FRIENDS_TABLE(NB_Friends,false) end
	end
end

function WFEZ_O:GET_FRIEND_TABLE(nb,bnet)
    if WFEZ_D.isOpen then
		if not nb then return end
        WFEZ_O:DEBUG('VAR','GET_FRIEND_TABLE')
		if bnet then WFEZ_O:GET_FRIENDS_BNET(nb) end
	end
end
function WFEZ_O:GET_FRIENDS_TABLE(nb,bnet)
    if WFEZ_D.isOpen then
        -- WFEZ_O:DEBUG('VAR','GET_FRIENDS_TABLE')
		if bnet then
			for i = 1, nb do WFEZ_O:GET_FRIENDS_BNET(i) end
		else
			for i = 1, nb do
				local info = C_FriendList.GetFriendInfoByIndex(i)
				tinsert(info, { app = 'IG'})
				if info then WFEZ_O:BUILD_FRIENDS(info,info.guid,'IG') end
			end
		end
    end
end

function WFEZ_O:GET_FRIENDS_BNET(i)
	local bnetinfo = C_BattleNet.GetFriendAccountInfo(i)
	if bnetinfo then

		local numGameAccounts = C_BattleNet.GetFriendNumGameAccounts(i)

		if numGameAccounts == 0 then ------------------------------- offline
			local build = { connected = false,
								 btag = bnetinfo.accountName, bnetid = bnetinfo.bnetAccountID,
								 name = bnetinfo.accountName,
								notes = bnetinfo.note }
			WFEZ_O:BUILD_FRIENDS(build,bnetinfo.bnetAccountID,'')
		else ------------------------------------------------------- online
			isapp = 0
			if numGameAccounts == 1 then
				local game = C_BattleNet.GetFriendGameAccountInfo(i, 1)
				local build = { connected = game.isOnline,
									 btag = bnetinfo.accountName, bnetid = bnetinfo.bnetAccountID,
									 name = game.characterName or bnetinfo.accountName,
								className = game.className,
									 race = game.raceName,
								realmName = game.realmName,
									level = game.characterLevel,
									 area = game.richPresence,
									  afk = game.isGameAFK,
									  dnd = game.isGameBusy,
								  faction = game.factionName,
									notes = bnetinfo.note }
				WFEZ_O:BUILD_FRIENDS(build,bnetinfo.bnetAccountID,game.clientProgram)
			else
				local toShow = {}
				local isApp = false
				local isMob = false
				for y = 1, numGameAccounts do
					local game = C_BattleNet.GetFriendGameAccountInfo(i, y)
						if game.clientProgram == "App"  then isApp = y
					elseif game.clientProgram == "BSAp" then isMob = y
					else tinsert(toShow,y) end
					-- WFEZ_O:DEBUG('VAR',bnetinfo.accountName..' - '..game.clientProgram..' '..(game.characterName or bnetinfo.accountName)..' #'..y)
				end
				if #toShow >= 1 then
					for _, y in pairs(toShow) do
						local game = C_BattleNet.GetFriendGameAccountInfo(i,y)
						local build = { connected = game.isOnline,
										 	 btag = bnetinfo.accountName, bnetid = bnetinfo.bnetAccountID,
											 name = game.characterName or bnetinfo.accountName,
										className = game.className,
										 	 race = game.raceName,
										realmName = game.realmName,
											level = game.characterLevel,
											 area = game.richPresence,
											  afk = game.isGameAFK,
											  dnd = game.isGameBusy,
										  faction = game.factionName,
											notes = bnetinfo.note }
						WFEZ_O:BUILD_FRIENDS(build,bnetinfo.bnetAccountID..(y > 1 and '-'..y or ''),game.clientProgram)
					end
				elseif isApp then
					local game = C_BattleNet.GetFriendGameAccountInfo(i,isApp)
					local build = { connected = game.isOnline,
										 btag = bnetinfo.accountName, bnetid = bnetinfo.bnetAccountID,
										 name = game.characterName or bnetinfo.accountName,
									className = game.className,
										 race = game.raceName,
									realmName = game.realmName,
										level = game.characterLevel,
										 area = game.richPresence,
										  afk = game.isGameAFK,
										  dnd = game.isGameBusy,
									  faction = game.factionName,
										notes = bnetinfo.note }
					WFEZ_O:BUILD_FRIENDS(build,bnetinfo.bnetAccountID,game.clientProgram)
				elseif isMob then
					local game = C_BattleNet.GetFriendGameAccountInfo(i,isMob)
					local build = { connected = game.isOnline,
										 btag = bnetinfo.accountName, bnetid = bnetinfo.bnetAccountID,
										 name = game.characterName or bnetinfo.accountName,
									className = game.className,
										 race = game.raceName,
									realmName = game.realmName,
										level = game.characterLevel,
										 area = game.richPresence,
										  afk = game.isGameAFK,
										  dnd = game.isGameBusy,
									  faction = game.factionName,
										notes = bnetinfo.note }
					WFEZ_O:BUILD_FRIENDS(build,bnetinfo.bnetAccountID,game.clientProgram)
				end
			end
		end
	end
end

function WFEZ_O:GET_GUILD_TABLE()
    if WFEZ_D.isOpen and IsInGuild() then
        WFEZ_O:DEBUG('VAR','GET_GUILD_TABLE')
		local Guild_RANK = {}
		local RosterGuild = {}
		for i=0, GuildControlGetNumRanks() do
			local NameRank = GuildControlGetRankName(i)
			Guild_RANK[i] = NameRank
		end
    	local totalMembers = GetNumGuildMembers()
        for i = 1, totalMembers do
            local name, rank, rankID, level, class, zone, note, _, isOnline, status, _, _, _, isMobile, _, _, guid = GetGuildRosterInfo(i)
            if name then
				local roleid = rankID..'/ '..rank
				if rankID == 0 then
					roleid = '1/ '..Guild_RANK[2]
				end
				if WFEZ_D.guild == "Pull N Wipe" and WFEZ_D.playernme == "Woøtzee" then
					if rankID == 2 then
						roleid = '1/ '..Guild_RANK[2]
					elseif rankID == 3 or rankID == 4 then
						roleid = '2/ '..Guild_RANK[4]..' / '..Guild_RANK[5]
					elseif rankID == 5 then roleid = '3/ '..Guild_RANK[6]
					elseif rankID == 6 then roleid = '4/ '..Guild_RANK[7]
					elseif rankID == 7 then roleid = '5/ '..Guild_RANK[8]
					elseif rankID == 8 then roleid = '6/ '..Guild_RANK[9]
					elseif rankID == 9 then roleid = '7/ '..Guild_RANK[10]
					end
				end
				local playername,playerrealm =  strsplit( "-",name, 2 ) or ""
				local build = { connected = isOnline,
									 btag = rank,
								   roleid = roleid,
							   guildindex = i,
									 name = playername or name,
								className = class,
								realmName = playerrealm,
									level = level,
									 area = zone,
									  afk = status == 1 and true or false,
									  dnd = status == 2 and true or false,
								  faction = WFEZ_D.englishFaction,
									notes = note }
				WFEZ_O:BUILD_FRIENDS(build,rankID..'_'..guid,'guild')
			end
        end
    end
end

function WFEZ_O:BUILD_FRIENDS(info,uniqid,app)
    if WFEZ_D.isOpen then
		local group = "xx NO GROUP xx"
		local inotes= info.notes or ""
		local nnotes=""
		local guild=""
		local function regexEscape(str) return str:gsub("[%(%)%.%%%+%-%*%?%[%^%$%]]", "%%%1") end

		if app == 'guild' then
			group = "# "..WFEZ_D.guild.." .... "..info.roleid
			nnotes = inotes
			app = "IG"
			guild=info.guildindex
		else
			local reg = inotes:match("#(.*)")
			if reg then group = reg end
			nnotes = inotes:gsub(regexEscape("#"..group),'')
		end

		if not WFEZ_D.friendsAl[group] then WFEZ_D.friendsAl[group] = {} end
		local color = WFEZ_O:GET_COLOR(info.className) or (app=='IG' and { r=.4,g=.4,b=.4 } or { r=.55,g=.8,b=.9 })

		local player = {
			uniqid 	= uniqid,
			name   	= info.name,
			roleid  = roleid,
			guild  	= guild,
			rio 	= WFEZ_O:GET_RIO(info.name,info.realmName or WFEZ_D.playerrlm ,info.faction),
			bnetid 	= info.bnetid or nil,
			app   	= app,
			btag 	= info.btag,
			online 	= info.connected,
			class   = info.className,
			race    = info.race,
			color  	= color,
			realm  	= (app == 'BSAp' and 'Mobile' or (info.realmName or '')),
			level  	= info.level and (info.level ~= WFEZ_D.maxlvl and (info.level ~= 0 and info.level or '') or '') or '',
			area   	= info.area,
			faction = info.faction,
			status 	= (info.afk and 'afk') or (info.dnd and 'dnd') or (info.connected and 'onl') or '',
			notes  	= nnotes
		}
		WFEZ_D.friendsAl[group][uniqid] = player
    	WFEZ_O:UPDATE_PLAYER(uniqid,player)
	end
end

-- -----------------------------------------------------------------------------
-- UPDATING
-- -----------------------------------------------------------------------------

function WFEZ_O:UPDATE_PLAYER(uniqid,player)
	local objn='WFEZFrnd_'..uniqid
	if _G[objn] then

		_G[objn]:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WFEZ_F.play_name_maxwidth, "ANCHOR_BOTTOMLEFT",-WFEZ_D.sizes.MARGES*2,WFEZ_D.fonts.TXT_07+1);
			GameTooltip:AddLine(player.name, player.color.r,player.color.g,player.color.b)
			GameTooltip:AddLine((player.race and player.race..' ' or '')..(player.class and player.class..' ' or '')..(player.faction and player.faction or ''), 1,1,1)
			GameTooltip:AddLine(player.notes, .6,.6,.6)
			GameTooltip:AddLine('-',.2,.2,.2)
			GameTooltip:AddLine(player.realm,1,1,1)
			GameTooltip:AddLine(player.area,.6,.6,.6)
			GameTooltip:Show()
			WFEZ_O:SHOW_RIO(GameTooltip,player.name,player.realm,player.faction)
		end)
		_G[objn]:SetScript("OnClick", function()
			if player.guild ~='' then -- wow guild
				local index = player.guild
				_G.GuildFrame_LoadUI()
				_G.SetGuildRosterSelection(index)
				_G.GuildFrame_Toggle()
				_G.GuildFrame:Show()
				_G.GuildFrameTab2:Click()
				_G.GuildMemberDetailFrame:Show()
				_G.SetGuildRosterSelection(index)
				_G.GuildMemberDetailFrame:ClearAllPoints()
				_G.GuildMemberDetailFrame:SetPoint("TOPLEFT",WFEZ_F.main,"TOPRIGHT",WFEZ_D.sizes.MARGES*3,0)
				_G.GuildMemberDetailFrame:SetParent(WFEZ_F.main)
			elseif player.app =='IG' then -- wow friend
				FriendsFrame_ShowDropdown(player.name, player.online, nil, nil, nil, 1);
			elseif player.bnetid then -- bnet friend
				FriendsFrame_ShowBNDropdown(player.btag, player.online, nil, nil, nil, 1, player.bnetid);
			end
		end)

		local invplayer = player.name..(player.realm ~='' and '-'..player.realm or '')
		local sbj = objn.."_invt";
		if _G[sbj] then
			if player.online and (player.app=='WoW' or player.app=='IG') then
				_G[sbj]:SetScript("OnClick", function()
    				if GetNumSubgroupMembers() > 1 and not UnitIsGroupLeader("player") then return end
    				if GetNumSubgroupMembers() == 5 then ConvertToRaid() end
					C_PartyInfo.InviteUnit(invplayer)
				end)
				_G[sbj]:SetAlpha(1)
				_G[sbj]:SetEnabled(true)
				_G[sbj].tooltipText = "Invite "..invplayer
			else
				_G[sbj]:SetAlpha(.2)
				_G[sbj]:SetEnabled(false)
			end
		end
		WFEZ_O:PRESENCE(objn,player.status,invplayer)

		local sbj = objn.."_wisp";
		if _G[sbj] then
			if player.online then
				if player.bnetid then -- bnet friend
					_G[sbj]:SetScript("OnClick", function() ChatFrame_SendBNetTell(player.btag) end)
					_G[sbj].tooltipText = "PM "..player.btag
				else
					_G[sbj].tooltipText = "PM "..invplayer
					_G[sbj]:SetScript("OnClick", function() SetItemRef( 'player:'..invplayer, format('|Hplayer:%1$s|h[%1$s]|h',invplayer), 'LeftButton' ) end)
				end
				_G[sbj]:SetAlpha(1)
				_G[sbj]:SetEnabled(true)
			else
				_G[sbj]:SetAlpha(.2)
				_G[sbj]:SetEnabled(false)
			end
		end

		local sbj = objn.."_on";   if _G[sbj] then
			if player.online then _G[sbj]:SetText('on') else _G[sbj]:SetText('off') end
		end
		local sbj = objn.."_app";  if _G[sbj] then _G[sbj]:SetText(player.app) end
		local sbj = objn.."_name"; if _G[sbj] then _G[sbj]:SetText(player.name)
												   _G[sbj]:SetTextColor(player.color.r,player.color.g,player.color.b,1) end
		local sbj = objn.."_srvr"; if _G[sbj] then _G[sbj]:SetText(player.realm)
												   _G[sbj]:SetTextColor(player.color.r,player.color.g,player.color.b,.4) end
		local sbj = objn.."_levl"; if _G[sbj] then _G[sbj]:SetText(player.level)
												   _G[sbj]:SetTextColor(player.color.r,player.color.g,player.color.b,.4) end
		local sbj = objn.."_btag"; if _G[sbj] then _G[sbj]:SetText(player.btag) end
		local sbj = objn.."_note"; if _G[sbj] then _G[sbj]:SetText(player.rio..WFEZ_O:TRIM(player.notes)..' |cff666666'..(player.area or '')) end
		local sbj = objn.."_fact"; if _G[sbj] then
			 if player.faction  == 'Alliance' then _G[sbj]:SetTexture(WFEZ_D.FOLDRFF.."PlusManz-Alliance")
			 elseif player.faction == 'Horde' then _G[sbj]:SetTexture(WFEZ_D.FOLDRFF.."PlusManz-Horde") else _G[sbj]:SetTexture('') end end

		local sbj = objn.."_icon"; if _G[sbj] then
				if player.app == 'IG'   then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-WoW")
			elseif player.app == 'WoW'  then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-WoW") 		--	World of Warcraft
			elseif player.app == 'S2'   then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-SC2")		--	StarCraft 2
			elseif player.app == 'D3'   then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-D3")		--	Diablo 3
			elseif player.app == 'WTCG' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-WTCG")		--	Hearthstone
			elseif player.app == 'Hero' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-HotS")		--	Heroes of the Storm
			elseif player.app == 'Pro'  then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-Overwatch")	--	Overwatch
			elseif player.app == 'CLNT' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-CHATICON-BLIZZ")
			elseif player.app == 'S1' 	then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-SC") 			--	StarCraft: Remastered
			elseif player.app == 'DST2' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-Destiny2")		--	Destiny 2
			elseif player.app == 'VIPR' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-CallOfDutyBlackOps4") --	Call of Duty: Black Ops 4
			elseif player.app == 'ODIN' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-CallOfDutyMWicon") --	Call of Duty: Modern Warfare
			elseif player.app == 'LAZR' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-CallOfDutyMW2icon") --	Call of Duty: Modern Warfare 2
			elseif player.app == 'ZEUS' then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-CallofDutyBlackOpsColdWaricon") --	Call of Duty: Black Ops Cold War
			elseif player.app == 'W3' 	then _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-Warcraft3Reforged") --	Warcraft III: Reforged
										else _G[sbj]:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-Battlenet") end
		end
	end
end

function WFEZ_O:PRESENCE(name,what,user)
    local objn= name.."_conn"
    if _G[objn] then
        if user and (UnitInParty(user) or UnitInRaid(user)) then _G[objn]:SetVertexColor( 0,.6,.9, 1)
    elseif what == 'mob' then _G[objn]:SetVertexColor(.8,.4, 0,.4)
    elseif what == 'afk' then _G[objn]:SetVertexColor(.8,.4, 0, 1)
    elseif what == 'dnd' then _G[objn]:SetVertexColor( 1, 0, 0, 1)
    elseif what == 'onl' then _G[objn]:SetVertexColor( 0,.9,.2, 1)
                         else _G[objn]:SetVertexColor( 1, 1, 1,.3) end
    end
end

-- -----------------------------------------------------------------------------
-- FRIENDS VIEW
-- -----------------------------------------------------------------------------

function WFEZ_O:CHECK_OPEN()
    if WFEZ_D.isOpen then WFEZ_F.main:Hide()
	else WFEZ_F.main:Show() end
end

function WFEZ_O:SET_FRIENDS()
    if WFEZ_D.isOpen then
        WFEZ_O:DEBUG('CALL','SET_FRIENDS ??')

		WFEZ_D.playernme,WFEZ_D.playerrlm  = UnitFullName("player")
		WFEZ_D.guild, _, _ = GetGuildInfo("player") or ''
		WFEZ_D.englishFaction, WFEZ_D.localizedFaction = UnitFactionGroup("player") or ''

		WFEZ_O:GET_FRIENDS()
        ------------------------------------------------------------------------
        ------------------------------------------------------------------ EMPTY
        local all_groups = { WFEZ_F.child:GetChildren() }
        for _, group in ipairs(all_groups) do
            if group:GetObjectType() == "CheckButton" then
                local groupName = group:GetName()
                   _G[groupName.."_txt"]:SetText('')

                local all_players = { group:GetChildren() }
                for _, player in ipairs(all_players) do
                    if player:GetObjectType() == "CheckButton" then
                        local playerName = player:GetName()
						_G[playerName.."_name"]:SetText('')
                    end
                end
            end
        end

        if WF_ez_DBC.enable_offs then WFEZ_F.bottnc:SetAlpha( 1)
		 						 else WFEZ_F.bottnc:SetAlpha(.4)  end

        if WF_ez_DBC.onlyig_offs then WFEZ_F.bottnd:SetAlpha( 1)
		 						 else WFEZ_F.bottnd:SetAlpha(.4)  end

        if WF_ez_DBC.showguildie then WFEZ_F.bottne:SetAlpha( 1)
		 						 else WFEZ_F.bottne:SetAlpha(.4)  end

        ------------------------------------------------------------------------

		local groupID = 0
		local groupOrder = { }
		for k in pairs(WFEZ_D.friendsAl) do tinsert(groupOrder, k) end
		table.sort(groupOrder)

		for i = 1, #groupOrder do
			local groupName = groupOrder[i]

			groupID = groupID + 1
			local grp = WFEZ_O:SET_LINE_group(groupID,groupName)
            -- ---------------------------------------------------------
            for k,p in pairs(WFEZ_D.friendsAl[groupName]) do
        	 	local play,toa = WFEZ_O:SET_LINE_play(grp,k,p.name,p.charact,p.color)
			 	WFEZ_O:UPDATE_PLAYER(k,WFEZ_D.friendsAl[groupName][k] )
        	end
		end
		WFEZ_O:REFRESH_FRIENDS()
    end ------------------------------------------------------------------------
end

function WFEZ_O:REFRESH_FRIENDS()
    if WFEZ_D.isOpen then
        WFEZ_O:DEBUG('CALL','REFRESH_FRIENDS ??')
		WFEZ_D.play_name_maxwidth = 80
        local hei, hei_grp,hei_pv = WFEZ_D.fonts.TTL_05, 0, 0
        local all_groups = { WFEZ_F.child:GetChildren() }
        for _, group in ipairs(all_groups) do
            if group:GetObjectType() == "CheckButton" then
                ----------------------------------------------------------------
                ----------------------------------------------------------------
                local groupName = group:GetName()
                local GName     = _G[groupName.."_txt"]:GetText()
                if GName then
                    local toadd = {}

                    local groupIDn = groupName:gsub("WFEZGroup_", "",1)
                    local groupID  = tonumber(groupIDn)
                  	group:SetPoint("TOPLEFT",0,"-"..hei)
                    group:SetAlpha(WF_ez_DBC.grouphidden[groupID] and .3 or 1)

                    hei,hei_grp = hei+WFEZ_D.sizes.lines_h,0

    		        local toadd_nbCheck,toadd_nb = 0,0
                    local all_players = { group:GetChildren() }
                    for _, player in ipairs(all_players) do
                        if player:GetObjectType() == "CheckButton" then
                            hei,hei_grp,toadd_nb,toadd_nbCheck,ntoadd = WFEZ_O:REFRESH_LINE_play(player,group,groupID,hei,hei_grp,toadd_nb,toadd_nbCheck)
                        end
                    end

                    if toadd_nb and toadd_nb > 0 then
						group:Show()
                        hei = hei+WFEZ_D.sizes.MMARGS
                    else
                        hei = (hei or 0)-WFEZ_D.sizes.lines_h
                        group:Hide()
                    end
                end ------------------------------------------------------------
            end
        end
        local Phei = WFEZ_F.scrollf:GetHeight()
        local max_hei = (hei-Phei<1 and 1 or hei-Phei)
        WFEZ_F.scrollb:SetMinMaxValues(1, max_hei)
		WFEZ_F.play_name_maxwidth:SetWidth(WFEZ_D.play_name_maxwidth)
        ------------------------------------------------------------------------
    end
end

function WFEZ_O:SET_LINE_group(i,name)
    local group = WFEZ_O:FRAM("WFEZGroup_"..i,"CheckButton",WFEZ_F.child,"ChatConfigCheckButtonTemplate",nil,nil,nil,nil,nil,WFEZ_D.fonts.TXT_14,WFEZ_D.fonts.TXT_14)
          group:DisableDrawLayer("BACKGROUND")
		  group:SetNormalTexture('') group:SetPushedTexture('') group:SetHighlightTexture('')
		  group:SetCheckedTexture('') group:SetDisabledCheckedTexture('')
          group:SetPoint("RIGHT",WFEZ_F.scrollf,"RIGHT",-WFEZ_D.sizes.MMARGS,0)

    local group_txt = WFEZ_O:FONT("WFEZGroup_"..i.."_txt",group,"LEFT",group,"LEFT",WFEZ_D.sizes.MARGES,0, nil,nil,WFEZ_D.fonts.TTL_F,"LEFT")
          group_txt:SetPoint("RIGHT",WFEZ_F.scrollf, "RIGHT",-(WFEZ_D.sizes.MARGES),0)
          group_txt:SetText(string.upper(name).." "..WFEZ_D.texte.points..WFEZ_D.texte.points..WFEZ_D.texte.points)
          group_txt:SetTextColor(1,1,1,1)

		  group:SetScript("OnEnter", function() group_txt:SetTextColor(.5,.9,1) end)
		  group:SetScript("OnLeave", function() group_txt:SetTextColor(1,1,1) end)
          group:SetScript("OnClick", function()
				if WF_ez_DBC.grouphidden[i] then
				   WF_ez_DBC.grouphidden[i] = nil
			  else WF_ez_DBC.grouphidden[i] = true end
			  WFEZ_O:REFRESH_FRIENDS()
		  end)

          group:Show()
    return group
end

-- -----------------------------------------------------------------------------

function WFEZ_O:SET_LINE_play(group,k,bnet,name,color)
    local layer_name = "WFEZFrnd_"..k

	-- print('- -> '..layer_name..' / '..bnet..' - '..name)
    local play = WFEZ_O:FRAM(layer_name,"CheckButton",WFEZ_F.child,"ChatConfigCheckButtonTemplate")
          play:SetNormalTexture('')  play:DisableDrawLayer("BACKGROUND")
		  play:SetPushedTexture('')  play:SetHighlightTexture('')
		  play:SetCheckedTexture('') play:SetDisabledCheckedTexture('')
		  play:SetParent(group)

		  play:SetScript("OnLeave", function() GameTooltip:Hide() end)

	-- ICONES ------------------------------------------------------------------

    local play_bble  = WFEZ_O:ICON(layer_name.."_conn",play,WFEZ_D.FOLDR .."round.tga","LEFT",play,"LEFT",WFEZ_D.fonts.TXT,WFEZ_D.fonts.TXT)
    local play_icon  = WFEZ_O:ICON(layer_name.."_icon",play,'',"LEFT",play_bble,"RIGHT")

	-- NAME + SERVER + LEVEL ---------------------------------------------------

    local play_on   = WFEZ_O:FONT(layer_name.."_on",play, "LEFT",play_icon,"RIGHT")
		  play_on:SetPoint("RIGHT",WFEZ_F.play_name_maxwidth,"RIGHT") play_on:SetAlpha(0)
    local play_name = WFEZ_O:FONT(layer_name.."_name",play,"LEFT",play_on,"LEFT",WFEZ_D.sizes.MMARGS,0)
		  play_name:SetText(name)
    local play_levl = WFEZ_O:FONT(layer_name.."_levl",play,"RIGHT",play_on,"RIGHT",0,0, nil,nil,WFEZ_D.fonts.TXT_F ,"RIGHT")
     local play_srvr = WFEZ_O:FONT(layer_name.."_srvr",play,"LEFT",play_name,"RIGHT",WFEZ_D.sizes.MMARGS,0)
	       play_srvr:SetPoint("RIGHT",play_levl,"LEFT",0,0)

	----------------------------------------------------------------------------

	local play_wisp = WFEZ_O:FRAM(layer_name.."_wisp","Button",play,"UIPanelButtonTemplate","LEFT",play_on,"RIGHT",WFEZ_D.sizes.MMARGS,0, WFEZ_D.fonts.TXT_14, WFEZ_D.fonts.TXT_14)
		  play_wisp:DisableDrawLayer("BACKGROUND") play_wisp:SetHighlightTexture('')
		  play_wisp:SetNormalTexture(WFEZ_D.FOLDRFF.."UI-Toast-BroadcastIcon")
		  play_wisp:SetPushedTexture(WFEZ_D.FOLDRFF.."UI-Toast-BroadcastIcon")
		  play_wisp:SetFrameStrata("HIGH")

	local play_invt = WFEZ_O:FRAM(layer_name.."_invt","Button",play,"UIPanelButtonTemplate","LEFT",play_wisp,"RIGHT",nil,nil, WFEZ_D.fonts.TXT_18, WFEZ_D.fonts.TXT_18)
		  play_invt:DisableDrawLayer("BACKGROUND") play_invt:SetHighlightTexture('')
		  play_invt:SetNormalTexture(WFEZ_D.FOLDRFF.."PlusManz-PlusManz")
		  play_invt:SetPushedTexture(WFEZ_D.FOLDRFF.."PlusManz-PlusManz")
		  play_invt:SetFrameStrata("HIGH")

	-- NOTES + AREA + BTAG -----------------------------------------------------

    local play_app = WFEZ_O:FONT(layer_name.."_app",play, "LEFT",play_invt,"RIGHT",0,0)
		  play_app:SetPoint("RIGHT",play,"RIGHT",0,0) play_app:SetAlpha(0)

	local play_fact = WFEZ_O:ICON(layer_name.."_fact",play,'',"LEFT",play_app,"RIGHT")

	local play_btag = WFEZ_O:FONT(layer_name.."_btag",play,"RIGHT",play_app,"RIGHT",0,0, nil,nil,WFEZ_D.fonts.TXT_F ,"RIGHT")
	      play_btag:SetTextColor(1,1,1,.4)

	local play_note = WFEZ_O:FONT(layer_name.."_note",play,"LEFT",play_app,"LEFT",WFEZ_D.sizes.MMARGS,0)
	      play_note:SetTextColor(1,1,1,.8)
	      play_note:SetPoint("RIGHT",play_btag,"LEFT",-WFEZ_D.sizes.MMARGS,0)

	----------------------------------------------------------------------------
    return play,name

end

function WFEZ_O:REFRESH_LINE_play(player,group,groupID,hei,hei_grp,toadd_nb,toadd_nbCheck)
    local layer_name = player:GetName() if  _G[layer_name.."_name"] then
    -- local checkname  = _G[layer_name.."_btag"]:GetText()
    local PName = _G[layer_name.."_name"]:GetText()
    local on    = _G[layer_name.."_on"]:GetText()
    local app   = _G[layer_name.."_app"]:GetText()
	local play_name_w = 0
    if PName then
		if WF_ez_DBC.onlyig_offs and (app ~= 'WoW' and app ~= 'IG') then
            player:Hide()

		elseif not WF_ez_DBC.enable_offs and on=='off' then
            player:Hide()
		else
			toadd_nb = toadd_nb+1
			if not WF_ez_DBC.grouphidden[groupID] then
        	    hei_grp = hei_grp+WFEZ_D.sizes.lines_h
        	    hei = hei+WFEZ_D.sizes.lines_h
        	    player:Show()

				play_name_w = _G[layer_name.."_name"]:GetStringWidth()+_G[layer_name.."_levl"]:GetStringWidth()+_G[layer_name.."_srvr"]:GetStringWidth()
				if play_name_w > WFEZ_D.play_name_maxwidth then WFEZ_D.play_name_maxwidth = play_name_w end
        	else
        	    player:Hide()
        	end
		end
		player:ClearAllPoints()
        player:SetParent(group)
		player:SetPoint("LEFT" ,group,"LEFT" ,  WFEZ_D.sizes.MMARGS     ,-(hei_grp))
		player:SetPoint("RIGHT",group,"RIGHT",-(WFEZ_D.sizes.MARGES*2.2),0)

        return hei,hei_grp,toadd_nb,toadd_nbCheck,PName
    else
        player:Hide()
        return hei,hei_grp,toadd_nb,toadd_nbCheck,nil
    end end
end

-- -----------------------------------------------------------------------------
-- WELCOME
-- -----------------------------------------------------------------------------

function WFEZ_O:SHOW_WELCOME()
    WFEZ_O:DEBUG('CALL','SHOW_WELCOME')
    local d_global = {
    }
    local defaults = {
        width = 444,
        groups = {},
        search = ""
    }
	local function copyDefaults(src, dst)
		if type(src) ~= "table" then return {} end
		if not type(dst) then dst = {} end
		for k, v in pairs(src) do
			if type(v) == "table" then
				dst[k] = copyDefaults(v, dst[k])
			elseif type(v) ~= type(dst[k]) then
				dst[k] = v
			end
		end
		return dst
	end
    WF_ez_DBC = copyDefaults(defaults, WF_ez_DBC)

    WFEZ_D.sizes.PANNEL = WF_ez_DBC.width
    WFEZ_F.main:SetWidth(WFEZ_D.sizes.PANNEL)

	WF_ez_DBC.grouphidden = WF_ez_DBC.grouphidden or {}
	WF_ez_DBC.UnlocalizedClasses = WF_ez_DBC.UnlocalizedClasses or {}
	for k, v in pairs(_G.LOCALIZED_CLASS_NAMES_MALE) do WF_ez_DBC.UnlocalizedClasses[v] = k end
	for k, v in pairs(_G.LOCALIZED_CLASS_NAMES_FEMALE) do WF_ez_DBC.UnlocalizedClasses[v] = k end
end



-- -----------------------------------------------------------------------------
-- FOR SCREENSHOT AND MORE
-- -----------------------------------------------------------------------------
function WFEZ_O:FAKE_GUILDIES()
	local classes = {"HUNTER", "WARLOCK", "PRIEST", "PALADIN", "MAGE", "ROGUE", "DRUID", "SHAMAN", "WARRIOR", "DEATHKNIGHT", "MONK", "DEMONHUNTER"};

	local guildies = ''
	local clssgul = {"MONK", "DEMONHUNTER", "DEMONHUNTER", "WARLOCK", "MAGE", "DEATHKNIGHT", "ROGUE", "SHAMAN", "WARRIOR" };
	local namegul = {"Nickname", "Guildie1", "Playerbeta", "Lidia", "Ramírez338", "Colomer", "Belen", "Shamyplayer", "Warhit"};

	for i, y in pairs(clssgul) do
		local c = RAID_CLASS_COLORS[y] or 'FFF4F4F4' -- WFEZ_O:GET_COLOR(className)
		guildies =guildies.." |c"..c.colorStr..namegul[i].."|r ";
	end
	return guildies

end
